/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2006 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.calendar.printing.osgi;

import static com.openexchange.calendar.printing.CPServiceRegistry.getInstance;
import java.util.Stack;
import org.osgi.framework.ServiceRegistration;
import org.osgi.util.tracker.ServiceTracker;
import com.openexchange.calendar.printing.CPServlet;
import com.openexchange.calendar.printing.preferences.CalendarPrintingEnabled;
import com.openexchange.group.GroupService;
import com.openexchange.groupware.calendar.AppointmentSqlFactoryService;
import com.openexchange.groupware.calendar.CalendarCollectionService;
import com.openexchange.groupware.settings.PreferencesItemService;
import com.openexchange.i18n.I18nService;
import com.openexchange.server.osgiservice.DeferredActivator;
import com.openexchange.server.osgiservice.RegistryCustomizer;
import com.openexchange.templating.TemplateService;
import com.openexchange.tools.servlet.http.HTTPServletRegistration;
import com.openexchange.user.UserService;

/**
 * {@link Activator} - The activator for calendar printing.
 * 
 * @author <a href="mailto:tobias.prinz@open-xchange.com">Tobias Prinz</a>
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public class Activator extends DeferredActivator {

    /**
     * The servlet path.
     */
    public static final String ALIAS = "/ajax/printCalendar";

    private HTTPServletRegistration registration;

    private ServiceRegistration preferenceItemRegistration;

    private Stack<ServiceTracker> trackers;

    @Override
    protected Class<?>[] getNeededServices() {
        return new Class<?>[] { TemplateService.class, AppointmentSqlFactoryService.class, CalendarCollectionService.class };
    }

    @Override
    protected void handleAvailability(final Class<?> clazz) {
        register();
    }

    @Override
    protected void handleUnavailability(final Class<?> clazz) {
        unregister();
    }

    @Override
    protected void startBundle() throws Exception {
        trackers = new Stack<ServiceTracker>();
        trackers.add(new ServiceTracker(context, I18nService.class.getName(), new I18nCustomizer(context)));
        trackers.add(new ServiceTracker(context, UserService.class.getName(), new RegistryCustomizer<UserService>(context, UserService.class, getInstance())));
        trackers.add(new ServiceTracker(context, GroupService.class.getName(), new RegistryCustomizer<GroupService>(context, GroupService.class, getInstance())));
        for (final ServiceTracker tracker : trackers) {
            tracker.open();
        }
        register();
        preferenceItemRegistration = context.registerService(PreferencesItemService.class.getName(), new CalendarPrintingEnabled(), null);
    }

    @Override
    protected void stopBundle() throws Exception {
        if (null != preferenceItemRegistration) {
            preferenceItemRegistration.unregister();
            preferenceItemRegistration = null;
        }
        unregister();
        if (null != trackers) {
            while (!trackers.isEmpty()) {
                trackers.pop().close();
            }
            trackers = null;
        }
    }

    private void register() {
        final TemplateService templates = getService(TemplateService.class);
        final AppointmentSqlFactoryService appointmentSqlFactory = getService(AppointmentSqlFactoryService.class);
        final CalendarCollectionService collectionService = getService(CalendarCollectionService.class);

        if (templates == null || appointmentSqlFactory == null || collectionService == null) {
            unregister();
            return;
        }

        CPServlet.setTemplateService(templates);
        CPServlet.setAppointmentSqlFactoryService(appointmentSqlFactory);
        CPServlet.setCalendarTools(collectionService);

        registration = new HTTPServletRegistration(context, ALIAS, new CPServlet());
    }

    private void unregister() {
        if (registration != null) {
            registration.unregister();
            registration = null;
        }
    }
}
